/* 
 * @HEADER
 *
 * ***********************************************************************
 *
 *  Zoltan Toolkit for Load-balancing, Partitioning, Ordering and Coloring
 *                  Copyright 2012 Sandia Corporation
 *
 * Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 * the U.S. Government retains certain rights in this software.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the Corporation nor the names of the
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Questions? Contact Karen Devine	kddevin@sandia.gov
 *                    Erik Boman	egboman@sandia.gov
 *
 * ***********************************************************************
 *
 * @HEADER
 */

#ifndef __RCB_PARAMS_H
#define __RCB_PARAMS_H

#ifdef __cplusplus
/* if C++, define the rest of this header file as extern C */
extern "C" {
#endif


#include "zz_const.h"
#include "params_const.h"

/*  Parameters structure for RCB method.  Used in  */
/*  Zoltan_RCB_Set_Param and Zoltan_RCB.                   */
static PARAM_VARS RCB_params[] = {
                  { "RCB_OVERALLOC", NULL, "DOUBLE", 0 },
                  { "RCB_REUSE", NULL, "INT", 0 },
                  { "CHECK_GEOM", NULL, "INT", 0 },
                  { "RCB_OUTPUT_LEVEL", NULL, "INT", 0 },
                  { "KEEP_CUTS", NULL, "INT", 0 },
                  { "RCB_LOCK_DIRECTIONS", NULL, "INT", 0 },
                  { "RCB_SET_DIRECTIONS", NULL, "INT", 0 },
                  { "RCB_RECTILINEAR_BLOCKS", NULL, "INT", 0 },
                  { "OBJ_WEIGHTS_COMPARABLE", NULL, "INT", 0 },
                  { "RCB_MULTICRITERIA_NORM", NULL, "INT", 0 },
                  { "RCB_MAX_ASPECT_RATIO", NULL, "DOUBLE", 0 },
                  { "AVERAGE_CUTS", NULL, "INT", 0 },
                  { "RANDOM_PIVOTS", NULL, "INT", 0 },
                  { "RCB_RECOMPUTE_BOX", NULL, "INT", 0 },
                  { "REDUCE_DIMENSIONS", NULL, "INT", 0 },
                  { "DEGENERATE_RATIO", NULL, "DOUBLE", 0 },
                  {"FINAL_OUTPUT",      NULL,  "INT",    0},
                  { NULL, NULL, NULL, 0 } };

#ifdef __cplusplus
} /* closing bracket for extern "C" */
#endif

#endif
